"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = (0, tslib_1.__importDefault)(require("chalk"));
const errors_1 = require("@oclif/parser/lib/errors");
const errors_2 = require("@oclif/errors");
const command_1 = (0, tslib_1.__importDefault)(require("../command"));
const flagsBuilder = (0, tslib_1.__importStar)(require("../flags"));
const definition_directory_1 = require("../core/definition_directory");
const deploy_1 = require("../core/deploy");
const prompts_1 = require("../core/utils/prompts");
const file_1 = require("../core/utils/file");
const args_1 = require("../args");
const cli_1 = require("../cli");
const definition_1 = require("../definition");
class Deploy extends command_1.default {
    /*
      Oclif doesn't type parsed args & flags correctly and especially
      required-ness which is not known by the compiler, thus the use of
      the non-null assertion '!' in this command.
      See https://github.com/oclif/oclif/issues/301 for details
    */
    async run() {
        const { args, flags } = this.parse(Deploy);
        const [dryRun, documentation, token, hub, autoCreate, interactive, filenamePattern, documentationName, branch,] = [
            flags['dry-run'],
            flags.doc,
            /* eslint-disable-next-line @typescript-eslint/no-non-null-assertion */
            flags.token,
            flags.hub,
            flags['auto-create'],
            flags.interactive,
            /* Flags.filenamePattern has a default value, so it's always defined. But
             * oclif types doesn't detect it */
            /* eslint-disable-next-line @typescript-eslint/no-non-null-assertion */
            flags['filename-pattern'],
            flags['doc-name'],
            flags.branch,
        ];
        if ((0, file_1.isDir)(args.FILE)) {
            if (hub) {
                await this.deployDirectory(args.FILE, dryRun, token, hub, autoCreate, interactive, filenamePattern, documentationName, branch);
            }
            else {
                throw new errors_1.RequiredFlagError({ flag: Deploy.flags.hub, parse: {} });
            }
        }
        else {
            if (documentation) {
                const api = await definition_1.API.load(args.FILE);
                this.d(`${args.FILE} looks like an ${api.specName} spec version ${api.version}`);
                await this.deploySingleFile(api, dryRun, documentation, token, hub, autoCreate, documentationName, branch);
            }
            else {
                throw new errors_1.RequiredFlagError({ flag: Deploy.flags.doc, parse: {} });
            }
        }
        return;
    }
    async deployDirectory(dir, dryRun, token, hub, autoCreate, interactive, filenamePattern, documentationName, branch) {
        const definitionDirectory = new definition_directory_1.DefinitionDirectory(dir, filenamePattern);
        const action = dryRun ? 'validate' : 'deploy';
        await definitionDirectory.readDefinitions();
        // In “interactive” mode we ask the user if he wants to add more
        // definitions to deploy. He is thus presented a form to select
        // some files from the target directory.
        if (interactive) {
            let confirm = true;
            if (definitionDirectory.definitionsExists()) {
                await (0, prompts_1.confirm)('Do you want to add more files to deploy?').catch(() => {
                    confirm = false;
                });
            }
            if (confirm) {
                await definitionDirectory.interactiveSelection();
            }
        }
        if (definitionDirectory.definitionsExists()) {
            cli_1.cli.info(chalk_1.default.underline(`Let's ${action} those documentations to your ${hub} hub on Bump.sh`));
            await definitionDirectory.sequentialMap(async (definition) => {
                if (interactive) {
                    await definitionDirectory.renameToConvention(definition);
                }
                await this.deploySingleFile(definition.definition, dryRun, definition.slug, token, hub, autoCreate, definition.slug || documentationName, branch);
                return definition;
            });
        }
        else {
            throw new errors_2.CLIError(`No documentations found in ${dir}.\nYou should check the ${chalk_1.default.dim('--filename-pattern')} flag to select your files from your naming convention.\nIf you don't have a naming convention we can help naming your API definition files:\nTry the ${chalk_1.default.dim('--interactive')} flag for that.`);
        }
        return;
    }
    async deploySingleFile(api, dryRun, documentation, token, hub, autoCreate, documentationName, branch) {
        const action = dryRun ? 'validate' : 'deploy';
        cli_1.cli.action.start(`Let's ${action} a new version to your ${documentation} documentation on Bump.sh`);
        const response = await new deploy_1.Deploy(this.config).run(api, dryRun, documentation, token, hub, autoCreate, documentationName, branch);
        if (dryRun) {
            await cli_1.cli.styledSuccess('Definition is valid');
        }
        else {
            if (response) {
                await cli_1.cli.styledSuccess(`Your new documentation version will soon be ready at ${response.doc_public_url}`);
            }
            else {
                await cli_1.cli.warn('Your documentation has not changed');
            }
        }
        cli_1.cli.action.stop();
        return;
    }
}
exports.default = Deploy;
Deploy.description = 'Create a new version of your documentation from the given file or URL.';
Deploy.examples = [
    `Deploy a new version of ${chalk_1.default.underline('an existing documentation')}

${chalk_1.default.dim('$ bump deploy FILE --doc <your_doc_id_or_slug> --token <your_doc_token>')}
* Let's deploy a new documentation version on Bump... done
* Your new documentation version will soon be ready
`,
    `Deploy a new version of ${chalk_1.default.underline('an existing documentation attached to a hub')}

${chalk_1.default.dim('$ bump deploy FILE --doc <doc_slug> --hub <your_hub_id_or_slug> --token <your_doc_token>')}
* Let's deploy a new documentation version on Bump... done
* Your new documentation version will soon be ready
`,
    `Deploy a whole directory of ${chalk_1.default.underline('API definitions files to a hub')}

${chalk_1.default.dim('$ bump deploy DIR --filename-pattern *-{slug}-api --hub <hub_slug> --token <hub_token>')}
We've found 2 valid API definitions to deploy
└─ DIR
   └─ source-my-service-api.yml (OpenAPI spec version 3.1.0)
   └─ source-my-jobs-service-api.yml (AsyncAPI spec version 2.6.0)

Let's deploy those documentations to your <hub_slug> hub on Bump.sh

* Your new documentation version will soon be ready
Let's deploy a new version to your my-service documentation on Bump.sh... done

* Your new documentation version will soon be ready
Let's deploy a new version to your my-jobs-service documentation on Bump.sh... done
`,
    `${chalk_1.default.underline('Validate a new documentation version')} before deploying it

${chalk_1.default.dim('$ bump deploy FILE --dry-run --doc <doc_slug> --token <your_doc_token>')}
* Let's validate a new documentation version on Bump... done
* Definition is valid
`,
];
Deploy.flags = {
    help: flagsBuilder.help({ char: 'h' }),
    doc: flagsBuilder.doc(),
    'doc-name': flagsBuilder.docName(),
    hub: flagsBuilder.hub(),
    branch: flagsBuilder.branch(),
    token: flagsBuilder.token(),
    'auto-create': flagsBuilder.autoCreate(),
    interactive: flagsBuilder.interactive(),
    'filename-pattern': flagsBuilder.filenamePattern(),
    'dry-run': flagsBuilder.dryRun(),
};
Deploy.args = [args_1.fileArg];
