"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const errors_1 = require("@oclif/errors");
const command_1 = (0, tslib_1.__importDefault)(require("../command"));
const flagsBuilder = (0, tslib_1.__importStar)(require("../flags"));
const diff_1 = require("../core/diff");
const args_1 = require("../args");
const cli_1 = require("../cli");
class Diff extends command_1.default {
    /*
      Oclif doesn't type parsed args & flags correctly and especially
      required-ness which is not known by the compiler, thus the use of
      the non-null assertion '!' in this command.
      See https://github.com/oclif/oclif/issues/301 for details
    */
    async run() {
        const { args, flags } = this.parse(Diff);
        /* Flags.format has a default value, so it's always defined. But
         * oclif types doesn't detect it */
        const [documentation, hub, branch, token, format, expires] = [
            flags.doc,
            flags.hub,
            flags.branch,
            flags.token,
            /* eslint-disable-next-line @typescript-eslint/no-non-null-assertion */
            flags.format,
            flags.expires,
        ];
        if (format === 'text') {
            if (args.FILE2) {
                cli_1.cli.action.start('* Comparing the two given definition files');
            }
            else {
                cli_1.cli.action.start('* Comparing the given definition file with the currently deployed one');
            }
        }
        if (!args.FILE2 && (!documentation || !token)) {
            throw new errors_1.CLIError('Please provide a second file argument or login with an existing token');
        }
        const diff = await new diff_1.Diff(this.config).run(args.FILE, args.FILE2, documentation, hub, branch, token, format, expires);
        cli_1.cli.action.stop();
        if (diff) {
            await this.displayCompareResult(diff, format, flags.open, flags['fail-on-breaking']);
        }
        else {
            await cli_1.cli.log('No changes detected.');
        }
        return;
    }
    async displayCompareResult(result, format, open, failOnBreaking) {
        if (format == 'text' && result.text) {
            await cli_1.cli.log(result.text);
        }
        else if (format == 'markdown' && result.markdown) {
            await cli_1.cli.log(result.markdown);
        }
        else if (format == 'json' && result.details) {
            await cli_1.cli.log(JSON.stringify(result.details, null, 2));
        }
        else if (format == 'html' && result.html) {
            await cli_1.cli.log(result.html);
        }
        else {
            await cli_1.cli.log('No structural changes detected.');
        }
        if (open && result.public_url) {
            await cli_1.cli.open(result.public_url);
        }
        if (failOnBreaking && result.breaking) {
            this.exit(1);
        }
    }
}
exports.default = Diff;
Diff.description = 'Get a comparison diff with your documentation from the given file or URL.';
Diff.examples = [
    `Compare a potential new version with the currently published one:

  $ bump diff FILE --doc <your_doc_id_or_slug> --token <your_doc_token>
  * Comparing the given definition file with the currently deployed one... done
  Removed: GET /compare
  Added: GET /versions/{versionId}
`,
    `Store the diff in a dedicated file:

  $ bump diff FILE --doc <doc_slug> --token <doc_token> > /tmp/my-saved-diff
  * Comparing the given definition file with the currently deployed one... done

  $ cat /tmp/my-saved-diff
  Removed: GET /compare
  Added: GET /versions/{versionId}
`,
    `In case of a non modified definition FILE compared to your existing documentation, no changes are output:

  $ bump diff FILE --doc <doc_slug> --token <your_doc_token>
  * Comparing the given definition file with the currently deployed one... done
   ›   Warning: Your documentation has not changed
`,
    `Compare two different input files or URL independently to the one published on bump.sh

  $ bump diff FILE FILE2 --doc <doc_slug> --token <your_doc_token>
  * Comparing the two given definition files... done
  Updated: POST /versions
    Body attribute added: previous_version_id
`,
];
Diff.flags = {
    help: flagsBuilder.help({ char: 'h' }),
    doc: flagsBuilder.doc(),
    hub: flagsBuilder.hub(),
    branch: flagsBuilder.branch(),
    token: flagsBuilder.token({ required: false }),
    open: flagsBuilder.open({ description: 'Open the visual diff in your browser' }),
    'fail-on-breaking': flagsBuilder.failOnBreaking(),
    format: flagsBuilder.format(),
    expires: flagsBuilder.expires(),
};
Diff.args = [args_1.fileArg, args_1.otherFileArg];
