"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const definition_1 = require("../definition");
const command_1 = (0, tslib_1.__importDefault)(require("../command"));
const flagsBuilder = (0, tslib_1.__importStar)(require("../flags"));
const args_1 = require("../args");
const cli_1 = require("../cli");
const fs_1 = require("fs");
const async_mutex_1 = require("async-mutex");
class Preview extends command_1.default {
    async run() {
        const { args, flags } = this.parse(Preview);
        await this.preview(args.FILE, flags.open);
        if (flags.live) {
            await this.waitForChanges(args.FILE, flags.open);
        }
        return;
    }
    async preview(file, open = false, currentPreview = undefined) {
        const api = await definition_1.API.load(file);
        const [definition, references] = api.extractDefinition();
        this.d(`${file} looks like an ${api.specName} spec version ${api.version}`);
        if (!currentPreview) {
            cli_1.cli.action.start("* Let's render a preview on Bump");
        }
        const request = {
            definition,
            references,
        };
        const response = currentPreview
            ? await this.bump.putPreview(currentPreview.id, request)
            : await this.bump.postPreview(request);
        if (!currentPreview) {
            cli_1.cli.action.stop();
            cli_1.cli.styledSuccess(`Your preview is visible at: ${response.data.public_url} (Expires at ${response.data.expires_at})`);
        }
        if (open && response.data.public_url) {
            await cli_1.cli.open(response.data.public_url);
        }
        return response.data;
    }
    async waitForChanges(file, open) {
        const mutex = new async_mutex_1.Mutex();
        let currentPreview = undefined;
        cli_1.cli.action.start(`Waiting for changes on file ${file}...`);
        (0, fs_1.watch)(file, async () => {
            if (!mutex.isLocked()) {
                const release = await mutex.acquire();
                const firstOpen = !currentPreview && open;
                this.preview(file, firstOpen, currentPreview)
                    .then((preview) => {
                    currentPreview = currentPreview || preview;
                    cli_1.cli.action.start(`Waiting for changes on file ${file}`);
                })
                    .catch((err) => {
                    this.warn(err);
                })
                    .finally(() => {
                    setTimeout(() => {
                        release();
                    }, 1000); // Prevent previewing faster than once per second
                });
            }
        });
    }
}
exports.default = Preview;
Preview.description = 'Create a documentation preview from the given file or URL.';
Preview.examples = [
    `$ bump preview FILE
* Your preview is visible at: https://bump.sh/preview/45807371-9a32-48a7-b6e4-1cb7088b5b9b
`,
];
Preview.flags = {
    help: flagsBuilder.help({ char: 'h' }),
    live: flagsBuilder.live({
        description: 'Generate a preview each time you save the given file',
    }),
    open: flagsBuilder.open({
        description: 'Open the generated preview URL in your browser',
    }),
};
Preview.args = [args_1.fileArg];
