"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefinitionDirectory = void 0;
const tslib_1 = require("tslib");
const chalk_1 = (0, tslib_1.__importDefault)(require("chalk"));
const debug_1 = (0, tslib_1.__importDefault)(require("debug"));
const fs_1 = require("fs");
const node_path_1 = require("node:path");
const p = (0, tslib_1.__importStar)(require("@clack/prompts"));
const errors_1 = require("@oclif/errors");
const cli_1 = require("../cli");
const definition_1 = require("../definition");
const file_1 = require("./utils/file");
const prompts_1 = require("../core/utils/prompts");
class DefinitionDirectory {
    constructor(directory, filenamePattern) {
        this.path = (0, node_path_1.resolve)(directory);
        this.definitions = [];
        // Transform basic patterns '*' or '{text}' into a real RegExp
        this.filenamePattern = new RegExp('^' + filenamePattern.replace('*', '.*?').replace(/{.*?}/, '(?<slug>.+?)') + '$');
        this.buildNewFilename = (slug) => filenamePattern.replace('*', '').replace(/{.*?}/, slug);
        this.humanFilenamePattern = filenamePattern.replace(/{(.*?)}/, `${chalk_1.default.inverse('{$1}')}`);
    }
    async readDefinitions() {
        var e_1, _a;
        try {
            for (var _b = (0, tslib_1.__asyncValues)(file_1.File.listValidConventionFiles(this.path, this.filenamePattern)), _c; _c = await _b.next(), !_c.done;) {
                const { value, filename } = _c.value;
                const file = (0, node_path_1.join)(this.path, value);
                /* We already check the filenamePattern match inside the
                `File.listValidConventionFiles` method so we are sure the group
                matched exists. */
                /* eslint-disable-next-line @typescript-eslint/no-non-null-assertion */
                const slug = filename.match(this.filenamePattern).groups.slug;
                const definition = await definition_1.API.load(file);
                this.definitions.push({
                    file,
                    definition,
                    slug,
                });
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) await _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        if (this.definitions.length) {
            cli_1.cli.info(chalk_1.default.underline(`We've found ${this.definitions.length} valid API definitions to deploy`));
            const subtree = cli_1.cli.tree();
            this.definitions.forEach(({ file, definition }) => subtree.insert(`${(0, node_path_1.basename)(file)} (${definition.specName} spec version ${definition.version})`));
            const tree = cli_1.cli.tree();
            tree.insert(this.path, subtree);
            tree.display();
            cli_1.cli.info('');
        }
        return this.definitions;
    }
    definitionsExists() {
        return !!this.definitions.length;
    }
    async sequentialMap(callback) {
        for (const definition of this.definitions) {
            await callback(definition);
        }
        return;
    }
    async renameToConvention(documentation) {
        const { file, slug } = documentation;
        if ((0, node_path_1.basename)(file, (0, node_path_1.extname)(file)).match(this.filenamePattern))
            return;
        // Default convention is defined in the flags.ts file for the
        // 'filenamePattern' flag.
        const newFilename = this.buildNewFilename(slug);
        const newFile = `${(0, node_path_1.dirname)(file)}/${newFilename}${(0, node_path_1.extname)(file)}`;
        let confirm = true;
        await (0, prompts_1.confirm)(`Do you want to rename ${file} to ${newFile} (for later deployments)?`).catch(() => {
            confirm = false;
        });
        if (confirm) {
            await (0, fs_1.rename)(file, newFile, (err) => {
                if (err)
                    throw err;
                cli_1.cli.styledSuccess(`Renamed ${file} to ${newFile}.`);
            });
        }
        return;
    }
    async interactiveSelection() {
        p.intro(`This interactive form will help you rename your API contrat files to follow the expected naming convention.\n${chalk_1.default.gray('│  ')}Once finished, the selected files will be deployed to Bump.sh.\n${chalk_1.default.gray('│  ')}\n${chalk_1.default.gray('│  ')}File naming convention: ${this.humanFilenamePattern}${chalk_1.default.dim('.[json|yml|yaml]')}\n`);
        const fileOptions = file_1.File.listInvalidConventionFiles(this.path, this.filenamePattern);
        if (!fileOptions.length) {
            throw new errors_1.CLIError(`No JSON or YAML files needing a rename were found in ${this.path}.\nAre you sure you need the ${chalk_1.default.dim('--interactive')} flag?`);
        }
        let shouldContinue = true;
        while (shouldContinue) {
            const filePrompt = {
                fileName: () => p.select({
                    message: `Which file do you want to deploy from ${chalk_1.default.dim(this.path)}?`,
                    options: fileOptions,
                }),
            };
            const groupPrompt = {
                /* Results type should be taken from the previous prompts
                 * defined with clack/prompt  */
                slug: ({ results }) => p.text({
                    message: `What is the ${chalk_1.default.inverse('documentation slug')} for this ${chalk_1.default.dim(results.fileName)} file?`,
                }),
            };
            const prompt = await p.group(Object.assign(Object.assign(Object.assign({}, filePrompt), groupPrompt), { shouldContinue: () => p.confirm({ message: 'Do you want to select another file?' }) }), {
                onCancel: () => {
                    p.cancel('Deploy cancelled.');
                    process.exit(0);
                },
            });
            const file = (0, node_path_1.join)(this.path, prompt.fileName);
            const definition = await definition_1.API.load(file);
            this.d(`${file} looks like an ${definition.specName} spec version ${definition.version}`);
            this.definitions.push({
                file,
                definition,
                slug: prompt.slug,
            });
            shouldContinue = prompt.shouldContinue;
        }
        p.outro(`You're all set. Your deployments will start soon.`);
        return this.definitions;
    }
    // Function signature type taken from @types/debug
    // Debugger(formatter: any, ...args: any[]): void;
    /* eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types, @typescript-eslint/no-explicit-any */
    d(formatter, ...args) {
        return (0, debug_1.default)(`bump-cli:core:interactive`)(formatter, ...args);
    }
}
exports.DefinitionDirectory = DefinitionDirectory;
