"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Diff = void 0;
const tslib_1 = require("tslib");
const errors_1 = require("@oclif/errors");
const debug_1 = (0, tslib_1.__importDefault)(require("debug"));
const definition_1 = require("../definition");
const api_1 = require("../api");
class Diff {
    constructor(config) {
        this._config = config;
    }
    async run(file1, file2, documentation, hub, branch, token, format, expires) {
        let diffVersion = undefined;
        if (file2 && (!documentation || !token)) {
            diffVersion = await this.createDiff(file1, file2, expires);
        }
        else {
            if (!documentation || !token) {
                throw new Error('Please login to bump (with documentation & token) when using a single file argument');
            }
            diffVersion = await this.createVersion(file1, documentation, token, hub, branch);
            if (file2) {
                diffVersion = await this.createVersion(file2, documentation, token, hub, branch, diffVersion && diffVersion.id);
            }
        }
        if (diffVersion) {
            return await this.waitResult(diffVersion, token, {
                timeout: 30,
                format,
            });
        }
        else {
            return undefined;
        }
    }
    get bumpClient() {
        if (!this._bump)
            this._bump = new api_1.BumpApi(this._config);
        return this._bump;
    }
    get pollingPeriod() {
        return 1000;
    }
    async createDiff(file1, file2, expires) {
        const api = await definition_1.API.load(file1);
        const [previous_definition, previous_references] = api.extractDefinition();
        const api2 = await definition_1.API.load(file2);
        const [definition, references] = api2.extractDefinition();
        const request = {
            previous_definition,
            previous_references,
            definition,
            references,
            expires_at: expires,
        };
        const response = await this.bumpClient.postDiff(request);
        switch (response.status) {
            case 201:
                this.d(`Diff created with ID ${response.data.id}`);
                this.d(response.data);
                return response.data;
                break;
            case 204:
                break;
        }
        return;
    }
    async createVersion(file, documentation, token, hub, branch_name, previous_version_id = undefined) {
        const api = await definition_1.API.load(file);
        const [definition, references] = api.extractDefinition();
        const request = {
            documentation,
            hub,
            definition,
            references,
            unpublished: true,
            previous_version_id,
            branch_name,
        };
        const response = await this.bumpClient.postVersion(request, token);
        switch (response.status) {
            case 201:
                this.d(`Unpublished version created with ID ${response.data.id}`);
                return response.data;
                break;
            case 204:
                break;
        }
        return;
    }
    async waitResult(result, token, opts) {
        let pollingResponse = undefined;
        if (this.isVersion(result) && token) {
            pollingResponse = await this.bumpClient.getVersion(result.id, token);
        }
        else {
            pollingResponse = await this.bumpClient.getDiff(result.id, opts.format);
        }
        if (opts.timeout <= 0) {
            throw new errors_1.CLIError('We were unable to compute your documentation diff. Sorry about that. Please try again later');
        }
        switch (pollingResponse.status) {
            case 200:
                let diff = pollingResponse.data;
                if (this.isVersionWithDiff(diff)) {
                    diff = this.extractDiff(diff);
                }
                this.d('Received diff:');
                this.d(diff);
                return diff;
                break;
            case 202:
                this.d('Waiting 1 sec before next poll');
                await this.pollingDelay();
                return await this.waitResult(result, token, {
                    timeout: opts.timeout - 1,
                    format: opts.format,
                });
                break;
        }
        return {};
    }
    async pollingDelay() {
        return await this.delay(this.pollingPeriod);
    }
    async delay(ms) {
        return new Promise((resolve) => setTimeout(resolve, ms));
    }
    // Function signature type taken from @types/debug
    // Debugger(formatter: any, ...args: any[]): void;
    /* eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types, @typescript-eslint/no-explicit-any */
    d(formatter, ...args) {
        return (0, debug_1.default)(`bump-cli:core:diff`)(formatter, ...args);
    }
    isVersion(result) {
        return result.doc_public_url !== undefined;
    }
    isVersionWithDiff(result) {
        return result.diff_summary !== undefined;
    }
    extractDiff(versionWithDiff) {
        // TODO: return a real diff_id in the GET /version API
        return {
            id: versionWithDiff.id,
            public_url: versionWithDiff.diff_public_url,
            text: versionWithDiff.diff_summary,
            markdown: versionWithDiff.diff_markdown,
            details: versionWithDiff.diff_details,
            breaking: versionWithDiff.diff_breaking,
        };
    }
}
exports.Diff = Diff;
