"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SupportedFormat = exports.API = void 0;
const tslib_1 = require("tslib");
const errors_1 = require("@oclif/errors");
const json_schema_ref_parser_1 = (0, tslib_1.__importDefault)(require("@apidevtools/json-schema-ref-parser"));
const options_1 = require("@apidevtools/json-schema-ref-parser/lib/options");
const specs_1 = (0, tslib_1.__importDefault)(require("@asyncapi/specs"));
const path_1 = (0, tslib_1.__importDefault)(require("path"));
class SupportedFormat {
}
exports.SupportedFormat = SupportedFormat;
SupportedFormat.openapi = {
    '2.0': require('oas-schemas/schemas/v2.0/schema.json'),
    '3.0': require('oas-schemas/schemas/v3.0/schema.json'),
    '3.1': require('oas-schemas/schemas/v3.1/schema.json'),
};
SupportedFormat.asyncapi = {
    '2.0': specs_1.default['2.0.0'],
    '2.1': specs_1.default['2.1.0'],
    '2.2': specs_1.default['2.2.0'],
    '2.3': specs_1.default['2.3.0'],
    '2.4': specs_1.default['2.4.0'],
    '2.5': specs_1.default['2.5.0'],
    '2.6': specs_1.default['2.6.0'],
};
class UnsupportedFormat extends errors_1.CLIError {
    constructor(message = '') {
        const compatOpenAPI = Object.keys(SupportedFormat.openapi).join(', ');
        const compatAsyncAPI = Object.keys(SupportedFormat.asyncapi).join(', ');
        const errorMsgs = [
            `Unsupported API specification (${message})`,
            `Please try again with an OpenAPI ${compatOpenAPI} or AsyncAPI ${compatAsyncAPI} file.`,
        ];
        super(errorMsgs.join('\n'));
    }
}
class API {
    constructor(location, $refs) {
        this.location = location;
        this.references = [];
        const [raw, parsed] = this.resolveContent($refs);
        this.rawDefinition = raw;
        this.definition = parsed;
        this.specName = this.getSpecName(parsed);
        this.version = this.getVersion(parsed);
        this.spec = this.getSpec(parsed);
        if (this.spec === undefined) {
            throw new UnsupportedFormat(`${this.specName} ${this.version}`);
        }
    }
    getSpec(definition) {
        if (API.isAsyncAPI(definition)) {
            return SupportedFormat.asyncapi[this.versionWithoutPatch()];
        }
        else {
            return SupportedFormat.openapi[this.versionWithoutPatch()];
        }
    }
    getSpecName(definition) {
        if (API.isAsyncAPI(definition)) {
            return 'AsyncAPI';
        }
        else {
            return 'OpenAPI';
        }
    }
    getVersion(definition) {
        if (API.isAsyncAPI(definition)) {
            return definition.asyncapi;
        }
        else {
            return (definition.openapi || definition.swagger);
        }
    }
    versionWithoutPatch() {
        const [major, minor] = this.version.split('.', 3);
        return `${major}.${minor}`;
    }
    /* Resolve reference absolute paths to the main api location when possible */
    resolveRelativeLocation(absPath) {
        const url = (location) => {
            try {
                return new URL(location);
            }
            catch (_a) {
                return { hostname: '' };
            }
        };
        const definitionUrl = url(this.location);
        const refUrl = url(absPath);
        if (absPath.match(/^\//) || // Unix style filesystem path
            absPath.match(/^[a-zA-Z]+\:\\/) || // Windows style filesystem path
            (absPath.match(/^https?:\/\//) && definitionUrl.hostname === refUrl.hostname) // Same domain URLs
        ) {
            return path_1.default.relative(path_1.default.dirname(this.location), absPath);
        }
        else {
            return absPath;
        }
    }
    resolveContent($refs) {
        const values = $refs.values();
        let mainReference = { parsed: {}, raw: '' };
        for (const [absPath, reference] of Object.entries(values)) {
            if (absPath === this.location || absPath === path_1.default.resolve(this.location)) {
                // $refs.values is not properly typed so we need to force it
                // with the resulting type of our custom defined parser
                mainReference = reference;
            }
            else {
                // $refs.values is not properly typed so we need to force it
                // with the resulting type of our custom defined parser
                const { raw } = reference;
                if (!raw) {
                    throw new UnsupportedFormat('Reference ${absPath} is empty');
                }
                this.references.push({
                    location: this.resolveRelativeLocation(absPath),
                    content: raw,
                });
            }
        }
        const { raw, parsed } = mainReference;
        if (!parsed || !(parsed instanceof Object) || !('info' in parsed)) {
            throw new UnsupportedFormat("Definition needs to be an object with at least an 'info' key");
        }
        if (!API.isOpenAPI(parsed) && !API.isAsyncAPI(parsed)) {
            throw new UnsupportedFormat();
        }
        return [raw, parsed];
    }
    static isOpenAPI(definition) {
        return (typeof definition.openapi === 'string' || typeof definition.swagger === 'string');
    }
    static isAsyncAPI(definition) {
        return 'asyncapi' in definition;
    }
    extractDefinition() {
        const references = [];
        for (let i = 0; i < this.references.length; i++) {
            const reference = this.references[i];
            references.push({
                location: reference.location,
                content: reference.content,
            });
        }
        return [this.rawDefinition, references];
    }
    static async load(path) {
        const JSONParser = options_1.defaults.parse.json;
        const YAMLParser = options_1.defaults.parse.yaml;
        const TextParser = options_1.defaults.parse.text;
        // We override the default parsers from $RefParser to be able
        // to keep the raw content of the files parsed
        const withRawTextParser = (parser) => {
            return Object.assign(Object.assign({}, parser), { parse: async (file) => {
                    const parsed = (await parser.parse(file));
                    return { parsed, raw: TextParser.parse(file) };
                } });
        };
        return json_schema_ref_parser_1.default
            .resolve(path, {
            parse: {
                json: withRawTextParser(JSONParser),
                yaml: withRawTextParser(YAMLParser),
                text: Object.assign(Object.assign({}, TextParser), { parse: async (file) => {
                        const parsed = await TextParser.parse(file);
                        return { parsed, raw: parsed };
                    }, canParse: ['.md', '.markdown'], encoding: 'utf8' }),
            },
            dereference: { circular: false },
        })
            .then(($refs) => {
            return new API(path, $refs);
        })
            .catch((err) => {
            throw new errors_1.CLIError(err);
        });
    }
}
exports.API = API;
